function job = cg_config_vbm
% Configuration file for VBM
%_______________________________________________________________________
%
% based on John Ashburners version of
% $Id: spm_config_preproc.m 424 2006-02-01 20:11:24Z john $
%
% @(#)cg_config_vbm.m	v1.18 Christian Gaser 2008/02/20

cg_vbm_defaults

tbs = spm('tbs');
for i=1:length(tbs)
	if strcmp(lower(tbs(i).name),'vbm5')
		addpath(tbs(i).dir);
	end
end

global defaults

entry = inline(['struct(''type'',''entry'',''name'',name,',...
        '''tag'',tag,''strtype'',strtype,''num'',num,''help'',{{}})'],...
        'name','tag','strtype','num');

files = inline(['struct(''type'',''files'',''name'',name,',...
        '''tag'',tag,''filter'',fltr,''num'',num,''help'',{{}})'],...
        'name','tag','fltr','num');

mnu = inline(['struct(''type'',''menu'',''name'',name,',...
        '''tag'',tag,''labels'',{labels},''values'',{values},''help'',{{}})'],...
        'name','tag','labels','values');

branch = inline(['struct(''type'',''branch'',''name'',name,',...
        '''tag'',tag,''val'',{val},''help'',{{}})'],...
        'name','tag','val');

%_______________________________________________________________________

data = files('Data','data','image',[1 Inf]);
data.help = {[...
'Select raw data (e.g. T1 images) for processing. ',...
'This assumes that there is one scan for each subject. ',...
'Note that multi-spectral (when there are two or more registered ',...
'images of different contrasts) processing is not yet implemented ',...
'for this method.']};

%------------------------------------------------------------------------
% various options for estimating the segmentations
%------------------------------------------------------------------------

tpm = files('Tissue probability maps','tpm','image',3);
tpm.dir = fullfile(spm('Dir'),'tpm');
tpm.def = 'vbm.opts.tpm';
tpm.help = {...
[...
'Select the tissue probability images. ',...
'These should be maps of grey matter, white matter ',...
'and cerebro-spinal fluid probability. ',...
'A nonlinear deformation field is estimated that best overlays the ',...
'tissue probability maps on the individual subjects'' image. ',...
'The default tissue probability maps are modified versions of the ',...
'ICBM Tissue Probabilistic Atlases.',...
'These tissue probability maps are kindly provided by the ',...
'International Consortium for Brain ',...
'Mapping, John C. Mazziotta and Arthur W. Toga. ',...
'http://www.loni.ucla.edu/ICBM/ICBM_TissueProb.html. ',...
'The original data are derived from 452 T1-weighted scans, ',...
'which were aligned with an atlas space, corrected for scan ',...
'inhomogeneities, and classified ',...
'into grey matter, white matter and cerebrospinal fluid. ',...
'These data were then affine registered to the MNI space and ',...
'downsampled to 2mm resolution.'],...
'',...
[...
'Rather than assuming stationary prior probabilities based upon mixing ',...
'proportions, additional information is used, based on other subjects'' brain ',...
'images.  Priors are usually generated by registering a large number of ',...
'subjects together, assigning voxels to different tissue types and averaging ',...
'tissue classes over subjects. ',...
'Three tissue classes are used: grey matter, white matter and cerebro-spinal fluid. ',...
'A fourth class is also used, which is simply one minus the sum of the first three. ',...
'These maps give the prior probability of any voxel in a registered image ',...
'being of any of the tissue classes - irrespective of its intensity.'],...
'',...
[...
'The model is refined further by allowing the tissue probability maps to be ',...
'deformed according to a set of estimated parameters. ',...
'This allows spatial normalisation and segmentation to be combined into ',...
'the same model. ',...
'This implementation uses a low-dimensional approach, which parameterises ',...
'the deformations by a linear combination of about a thousand cosine ',...
'transform bases. ',...
'This is not an especially precise way of encoding deformations, but it ',...
'can model the variability of overall brain shape. ',...
'Evaluations by Hellier et al have shown that this simple model can achieve ',...
'a registration accuracy comparable to other fully automated methods with ',...
'many more parameters.']};

%------------------------------------------------------------------------
ngaus      = entry('Gaussians per class','ngaus','n',[4 1]);
ngaus.def  = 'vbm.opts.ngaus';
ngaus.help = {[...
'The number of Gaussians used to represent the intensity distribution ',...
'for each tissue class can be greater than one. ',...
'In other words, a tissue probability map may be shared by several clusters. ',...
'The assumption of a single Gaussian distribution for each class does not ',...
'hold for a number of reasons. ',...
'In particular, a voxel may not be purely of one tissue type, and instead ',...
'contain signal from a number of different tissues (partial volume effects). ',...
'Some partial volume voxels could fall at the interface between different ',...
'classes, or they may fall in the middle of structures such as the thalamus, ',...
'which may be considered as being either grey or white matter. ',...
'Various other image segmentation approaches use additional clusters to ',...
'model such partial volume effects. ',...
'These generally assume that a pure tissue class has a Gaussian intensity ',...
'distribution, whereas intensity distributions for partial volume voxels ',...
'are broader, falling between the intensities of the pure classes. ',...
'Unlike these partial volume segmentation approaches, the model adopted ',...
'here simply assumes that the intensity distribution of each class may ',...
'not be Gaussian, and assigns belonging probabilities according to these ',...
'non-Gaussian distributions. ',...
'Typical numbers of Gaussians could be two for grey matter, two for white ',...
'matter, two for CSF, and four for everything else.']};

%------------------------------------------------------------------------

warpreg      = entry('Warping Regularisation','warpreg','e',[1 1]);
warpreg.def  = 'vbm.opts.warpreg';
warpreg.help = {[...
'The objective function for registering the tissue probability maps to the ',...
'image to process, involves minimising the sum of two terms. ',...
'One term gives a function of how probable the data is given the warping parameters. ',...
'The other is a function of how probable the parameters are, and provides a ',...
'penalty for unlikely deformations. ',...
'Smoother deformations are deemed to be more probable. ',...
'The amount of regularisation determines the tradeoff between the terms. ',...
'Pick a value around one.  However, if your normalized images appear ',...
'distorted, then it may be an idea to increase the amount of ',...
'regularization (by an order of magnitude). ',...
'More regularisation gives smoother deformations, ',...
'where the smoothness measure is determined by the bending energy of the deformations. ']};
%------------------------------------------------------------------------

warpco      = entry('Warp Frequency Cutoff','warpco','e',[1 1]);
warpco.def  = 'vbm.opts.warpco';
warpco.help = {[...
'Cutoff of DCT bases.  Only DCT bases of periods longer than the ',...
'cutoff are used to describe the warps. The number actually used will ',...
'depend on the cutoff and the field of view of your image. ',...
'A smaller cutoff frequency will allow more detailed deformations ',...
'to be modelled, but unfortunately comes at a cost of greatly increasing ',...
'the amount of memory needed, and the time taken.']};

%------------------------------------------------------------------------

biasreg = mnu('Bias regularisation','biasreg',{...
'no regularisation (0)','extremely light regularisation (0.00001)',...
'very light regularisation (0.0001)','light regularisation (0.001)',...
'medium regularisation (0.01)','heavy regularisation (0.1)',...
'very heavy regularisation (1)','extremely heavy regularisation (10)'},...
{0, 0.00001, 0.0001, 0.001, 0.01, 0.1, 1.0, 10});
biasreg.def  = 'vbm.opts.biasreg';
biasreg.help = {[...
'MR images are usually corrupted by a smooth, spatially varying artifact that modulates the intensity ',...
'of the image (bias). ',...
'These artifacts, although not usually a problem for visual inspection, can impede automated ',...
'processing of the images.'],...
'',...
[...
'An important issue relates to the distinction between intensity variations that arise because of ',...
'bias artifact due to the physics of MR scanning, and those that arise due to different tissue ',...
'properties.  The objective is to model the latter by different tissue classes, while modelling the ',...
'former with a bias field. ',...
'We know a priori that intensity variations due to MR physics tend to be spatially smooth, ',...
'whereas those due to different tissue types tend to contain more high frequency information. ',...
'A more accurate estimate of a bias field can be obtained by including prior knowledge about ',...
'the distribution of the fields likely to be encountered by the correction algorithm. ',...
'For example, if it is known that there is little or no intensity non-uniformity, then it would be wise ',...
'to penalise large values for the intensity nonuniformity parameters. ',...
'This regularisation can be placed within a Bayesian context, whereby the penalty incurred is the negative ',...
'logarithm of a prior probability for any particular pattern of nonuniformity.']};

%------------------------------------------------------------------------

biasfwhm    = mnu('Bias FWHM','biasfwhm',{...
'30mm cutoff','40mm cutoff','50mm cutoff','60mm cutoff','70mm cutoff',...
'80mm cutoff','90mm cutoff','100mm cutoff','110mm cutoff','120mm cutoff',...
'130mm cutoff','140mm cutoff','150mm cutoff','No correction'},...
{30,40,50,60,70,80,90,100,110,120,130,140,150,Inf});
biasfwhm.def  = 'vbm.opts.biasfwhm';
biasfwhm.help = {[...
'FWHM of Gaussian smoothness of bias. ',...
'If your intensity nonuniformity is very smooth, then choose a large ',...
'FWHM. This will prevent the algorithm from trying to model out intensity variation ',...
'due to different tissue types. The model for intensity nonuniformity is one ',...
'of i.i.d. Gaussian noise that has been smoothed by some amount, ',...
'before taking the exponential. ',...
'Note also that smoother bias fields need fewer parameters to describe them. ',...
'This means that the algorithm is faster for smoother intensity nonuniformities.']};

%------------------------------------------------------------------------

regtype = mnu('Affine Regularisation','regtype',...
   {'No Affine Registration','ICBM space template - European brains',...
    'ICBM space template - East Asian brains', 'Average sized template','No regularisation'},...
   {'','mni','eastern','subj','none'});
regtype.def  = 'vbm.opts.regtype';
regtype.help = {[...
'The procedure is a local optimisation, so it needs reasonable initial ',...
'starting estimates. Images should be placed in approximate alignment ',...
'using the Display function of SPM before beginning. ',...
'A Mutual Information affine registration with the tissue ',...
'probability maps (D''Agostino et al, 2004) is used to achieve ',...
'approximate alignment. ',...
'Note that this step does not include any model for intensity nonuniformity. ',...
'This means that if the procedure is to be initialised with the affine ',...
'registration, then the data should not be too corrupted with this artifact.',...
'If there is a lot of intensity nonuniformity, then manually position your ',...
'image in order to achieve closer starting estimates, and turn off the ',...
'affine registration.'],...
'',...
[...
'Affine registration into a standard space can be made more robust by ',...
'regularisation (penalising excessive stretching or shrinking).  The ',...
'best solutions can be obtained by knowing the approximate amount of ',...
'stretching that is needed (e.g. ICBM templates are slightly bigger ',...
'than typical brains, so greater zooms are likely to be needed). ',...
'For example, if registering to an image in ICBM/MNI space, then choose this ',...
'option.  If registering to a template that is close in size, then ',...
'select the appropriate option for this.']};

%------------------------------------------------------------------------

samp      = entry('Sampling distance','samp','e',[1 1]);
samp.def  = 'vbm.opts.samp';
samp.help = {[...
'The approximate distance between sampled points when estimating the ',...
'model parameters. Smaller values use more of the data, but the procedure ',...
'is slower.']};

%------------------------------------------------------------------------
% This field can be added to opts, by replacing a line below, with:
msk = files('Masking image','msk','image',[0 1]);
msk.def  = 'vbm.opts.msk';
msk.help = {[...
'The segmentation can be masked by an image that conforms to ',...
'the same space as the images to be segmented.  If an image is selected, then ',...
'it must match the image(s) voxel-for voxel, and have the same ',...
'voxel-to-world mapping.  Regions containing a value of zero in this image ',...
'do not contribute when estimating the various parameters. ']};

%------------------------------------------------------------------------

usecom    = mnu('Set origin','usecom',{...
'use origin from header','use center of mass to set origin'},...
{0,1});
usecom.def = 'vbm.opts.usecom';
usecom.help = {[...
'It is often helpful to set the origin (anterior comissure) using the Display tool ',...
' before spatial registration. However, the origin can be roughly approximated by calculating the center-of-mass (COM) ',...
'of the image and to shift the image to the COM of the template. This roughly corrects differences in the ',...
'position between the image and the template and utilizes spatial normalisation. In case that spatial registration ',...
'(and segmentation) fails with this options the safest way is to define the anterior comissure before segmentation ',...
'and to set this option to "use origin from header".']};

%------------------------------------------------------------------------
opts      = branch('Estimation options','opts',{tpm,ngaus,regtype,warpreg,warpco,biasreg,biasfwhm,samp,msk,usecom});
opts.help = {[...
'Various options can be adjusted in order to improve the performance of the ',...
'algorithm with your data.  Knowing what works best should be a matter ',...
'of empirical exploration.  For example, if your data has very little ',...
'intensity nonuniformity artifact, then the bias regularisation should ',...
'be increased.  This effectively tells the algorithm that there is very little ',...
'bias in your data, so it does not try to model it.']};

%_______________________________________________________________________
% options for output
%-----------------------------------------------------------------------

bb      = entry('Bounding box','bb','e',[2 3]);
bb.def = 'vbm.output.extopts.bb';
bb.help = {[...
'The bounding box (in mm) of the volume which is to be written ',...
'(relative to the anterior commissure). This is the default SPM5 value, but the ',...
'inferior z-border was lowered to -70mm to cover the whole cerebellum.']};

%------------------------------------------------------------------------

vox      = entry('Voxel sizes for normalisation','vox','e',[1 3]);
vox.def  = 'vbm.output.extopts.vox';
vox.help = {'The voxel sizes (x, y & z, in mm) of the written normalised images.'};

%------------------------------------------------------------------------

cleanup    = mnu('Clean up any partitions','cleanup',{'no Cleanup','Light Clean','Thorough Clean'},...
{0 1 2});
cleanup.def  = 'vbm.output.extopts.cleanup';
cleanup.help = {[...
'This uses a crude routine for extracting the brain from segmented',...
'images.  It begins by taking the white matter, and eroding it a',...
'couple of times to get rid of any odd voxels.  The algorithm',...
'continues on to do conditional dilations for several iterations,',...
'where the condition is based upon gray or white matter being present.',...
'This identified region is then used to clean up the grey and white',...
'matter partitions, and has a slight influences on the CSF partition.'],'',[...
'If you find pieces of brain being chopped out in your data, then you ',...
'may wish to disable or tone down the cleanup procedure.']};

%------------------------------------------------------------------------

mrf    = mnu('HMRF weighting','mrf',{...
'no HMRF','small HMRF weighting (0.15)','medium HMRF weighting (0.3)',...
'large HMRF weighting (0.45)','iterative HMRF weighting'},...
{0,0.15,0.3,0.45,1});
mrf.def  = 'vbm.output.extopts.mrf';
mrf.help = {[...
'A Hidden Markov Random Field (HMRF) is used to encode spatial information ',...
'through spatial constraints of neighboring voxels (Zhang et al. IEEE TMI 2001). ',...
'Neighboring voxels are expected to have the same class labels. The prior probability ',...
'of the class and the likelihood probability of the observation is combined to ',...
'estimate the Maximum a posteriori (MAP). Prior probability will be iteratively ',...
'weighted to cover a wide range of different noise levels. The fix HMRF weighting ',...
'was retained for compatibiliy purposes. The iterative weighting is the recommended ',...
'option and the resulting images are indicated by the term ''_aMRF'', whereas the fix ',...
'weighting is indicated by ''_HMRF''. Parts of this version are based on an ',...
'implementation of a HMRF by Meritxell Bach Cuadra et al. (IEEE TMI 2005).']};

%------------------------------------------------------------------------

usepriors    = mnu('Use tissue priors (experimental)','usepriors',{...
'no priors (new approach)','use priors (old SPM5 approach)'},...
{0,1});
usepriors.def    = 'vbm.output.extopts.usepriors';
usepriors.help = {[...
'SPM5 uses priors with a Bayes rule to combine the likelihood for belonging ',...
'to a tissue class and the prior probablity derived from prior probability maps. ',...
'However, the use of priors might cause problems for brains deviating from a ',...
'healthy, adult control population. The ICBM priors in SPM5 are based ',...
'on brains of subjects with an mean age of 25 years. Thus, brains of children or ',...
'or elderly subjects will deviate from the tissue probabilities of the ICBM priors ',...
'and will bias the segmentation. This options avoids the dependency on tissue priors ',...
'and usually increases classification accuracy. Segmented images show clearer delineation ',...
'particularly in the basal ganglia and in the sulci and this is the prefered option for most ',...
'images. If no priors are used the segmented images are indicated by an "p" instead of "c".'],...
'',...
[...
'Keep in mind that even if you are using this option tissue priors are always used ',...
'to register your images to MNI space. Only the bayesian rule to combine priors ',...
'with tissue type probabilities will be omitted. Thus, for some cases the use of ',...
'customized templates might utilize the spatial registration (e.g. infants). '],...
'',...
[...
'However, there are some cases (e.g. huge atrophy), where this option fails and ',...
'gray matter distribution might be underestimated. You will notice these cases by a very thin ',...
'gray matter ribbon and overestimated CSF. For those cases the default SPM5 tissue priors ',...
'(or own priors) should be used.'],...
};

%------------------------------------------------------------------------

writeaffine    = mnu('Write affine only','writeaffine',{...
'use warps and affine for writing data','use only affine for writing data'},{0,1});
writeaffine.def    = 'vbm.output.extopts.writeaffine';
writeaffine.help = {[...
'To obtain high segmentation accuracy its recommended to use the default warp frequency cutoff ',...
'of 25mm. This guarantees sufficient overlap between tissue priors and data. However, sometimes ',...
'you would like to save data using affine registering only while still retain high segmentation ',...
'accuracy. In this case you can internally apply warping for estimating segmentations, but write ',...
'data using affine registration only.']};

%------------------------------------------------------------------------

print    = mnu('Display and print results','print',{'yes','no'},{1,0});
print.def    = 'vbm.output.extopts.print';
print.help = {[...
'The normalized T1 image and the normalized segmentations can be displayed and printed to a ',...
'ps-file. This is often helpful to check whether registration and segmentation were successful.']};

%------------------------------------------------------------------------

extopts      = branch('Extended options','extopts',{usepriors,mrf,cleanup,vox,bb,writeaffine, print});
extopts.help = {'Extended options'};

%------------------------------------------------------------------------

native    = mnu('Native space','native',{'no','yes'},{0,1});
native.val  = {defaults.vbm.output.bias(1)};
native.help = {'Write bias corrected image in native space.'};

warped    = mnu('Normalized space','warped',{'no','yes'},{0,1});
warped.val  = {defaults.vbm.output.bias(2)};
warped.help = {'Write bias corrected image in normalized space.'};

descalp    = mnu('Additional scalp editing','descalp',{'no','yes'},{0,1});
descalp.val  = {defaults.vbm.output.bias(3)};
descalp.help = {'Write bias corrected image without scalp and skull.'};

bias      = branch('Bias Corrected','BIAS',{native, warped, descalp});
bias.help = {[...
'This is the option to save a bias corrected version of your image. ',...
'MR images are usually corrupted by a smooth, spatially varying artifact that modulates the intensity ',...
'of the image (bias). ',...
'These artifacts, although not usually a problem for visual inspection, can impede automated ',...
'processing of the images.  The bias corrected version should have more uniform intensities within ',...
'the different types of tissues and can be saved in native space and/or normalised. ',...
'Optionally the image can be saved without scalp and skull, which might be useful for external ',...
'software for cortical surface extraction (e.g. freesurfer). Keep in mind to write the data',...
'using affine transformation only to not deform the brain for this purpose.']};

%------------------------------------------------------------------------

native    = mnu('Native space','native',{'no','yes'},{0,1});
native.val  = {defaults.vbm.output.grey(1)};
native.help = {'Write segmented image in native space.'};

warped    = mnu('Unmodulated normalized','warped',{'no','yes'},{0,1});
warped.val  = {defaults.vbm.output.grey(2)};
warped.help = {'Write segmented image in normalized space.'};

modulated    = mnu('Modulated normalized','modulated',{'none','affine + non-linear (SPM5 default)','non-linear only'},{0,1,2});
modulated.val  = {defaults.vbm.output.grey(3)};
modulated.help = {[...
'Modulation is to compensate for the effect of spatial normalisation. Spatial normalisation ',...
'causes volume changes due to affine transformation (global scaling) and non-linear warping (local volume change). ',...
'The SPM default is to adjust spatially normalised grey matter (or other tissue class) by using both terms and the ',...
'resulting modulated images are preserved for the total amount of grey matter. Thus, modulated images reflect the grey matter ',...
'volumes before spatial normalisation. However, the user is often interested in removing the confound of different brain sizes ',...
'and there are many ways to apply this correction. We can use the total amount of GM, GM+WM, GM+WM+CSF, or manual estimated ',...
'total intracranial volume (TIV). Theses parameters can be modeled as nuisance parameters (additive effects) in an AnCova model ',...
'or used to globally scale the data (multiplicative effects): '],...
'',...
'% Correction   Interpretation',...
'% ----------   --------------',...
'% nothing      absolute volume',...
'% globals 	     relative volume after correcting for total GM or TIV (multiplicative effects)',...
'% AnCova 	      relative volume that can not be explained by total GM or TIV (additive effects)',...
'',...
[...
'I suggest another option to remove the confounding effects of different brain sizes. Modulated images can be optionally saved ',...
'by correcting for non-linear warping only. Volume changes due to affine normalisation will be not considered and this equals ',...
'the use of default modulation and globally scaling data according to the inverse scaling factor due to affine normalisation. I recommend ',...
'this option if your hypothesis is about effects of relative volumes which are corrected for different brain sizes. This is a widely ',...
'used hypothesis and should fit to most data. The idea behind this option is that scaling of affine normalisation is indeed a ',...
'multiplicative (gain) effect and we rather apply this correction to our data and not to our statistical model. ',...
'These modulated images are indicated by "m0" instead of "m". ']};

grey          = branch('Grey matter','GM',{native, warped, modulated});
grey.help     = {'Options to produce grey matter images: [cp]1*.img, w[cp]1*.img and mw[cp]1*.img.'};
native.val    = {defaults.vbm.output.white(1)};
warped.val    = {defaults.vbm.output.white(2)};
modulated.val = {defaults.vbm.output.white(3)};
white         = branch('White matter','WM',{native, warped, modulated});
white.help    = {'Options to produce white matter images: [cp]2*.img, w[cp]2*.img and mw[cp]2*.img.'};
native.val    = {defaults.vbm.output.csf(1)};
warped.val    = {defaults.vbm.output.csf(2)};
modulated.val = {defaults.vbm.output.csf(3)};
csf         = branch('Cerebro-Spinal Fluid (CSF)','CSF',{native, warped, modulated});
csf.help      = {'Options to produce CSF images: [cp]3*.img, w[cp]3*.img and mw[cp]3*.img.'};

%------------------------------------------------------------------------

output     = branch('Writing options','output',{grey,white,csf,bias,extopts});
output.help = {[...
'This routine produces spatial normalisation parameters (*_seg_sn.mat files) by default. ',...
'These can be used for writing spatially normalised versions of your data, via the "Normalise: Write" option. ',...
'This mechanism may produce superior results than the "Normalise: Estimate" option, although ',...
'this may need some empirical evaluations.'],...
'',...
[...
'In addition, it also produces files that can be used for doing inverse normalisation. ',...
'If you have an image of regions defined in the standard space, then the inverse deformations ',...
'can be used to warp these regions so that it approximately overlay your image. ',...
'To use this facility, the bounding-box and voxel sizes should be set to non-finite values ',...
'(e.g. [NaN NaN NaN] for the voxel sizes, and ones(2,3)*NaN for the bounding box. ',...
'This would be done by the spatial normalisation module, which allows you to select a ',...
'set of parameters that describe the nonlinear warps, and the images that they should be applied to.'],...
'',...
[...
'There are a number of options about what data you would like the routine to produce. ',...
'The routine can be used for producing images of tissue classes, as well as bias corrected images. ',...
'The native space option will produce a tissue class image (c*) that is in alignment with ',...
'the original/* (see Figure \ref{seg1})*/.  You can also produce spatially normalised versions - both',...
'with (mwc*) and without (wc*) modulation/* (see Figure \ref{seg2})*/. In contrast to Johns version',...
'the voxel sizes of the spatially normalised versions can be choosen, an HMRF can be applied, and the',...
'use of priors with the Bayes rule can be omitted.',...
'These can be used for doing voxel-based morphometry with (both un-modulated and modulated). ',...
'All you need to do is smooth them and do the stats (which means no more questions on the mailing list ',...
'about how to do "optimized VBM").'],...
'',...
[...
'Modulation is to compensate for the effect of spatial normalisation.  When warping a series ',...
'of images to match a template, it is inevitable that volumetric differences will be introduced ',...
'into the warped images.  For example, if one subject''s temporal lobe has half the volume of that of ',...
'the template, then its volume will be doubled during spatial normalisation. This will also ',...
'result in a doubling of the voxels labeled grey matter.  In order to remove this confound, the ',...
'spatially normalised grey matter (or other tissue class) is adjusted by multiplying by its relative ',...
'volume before and after warping.  If warping results in a region doubling its volume, then the ',...
'correction will halve the intensity of the tissue label. This whole procedure has the effect of preserving ',...
'the total amount of grey matter signal in the normalised partitions.'],...
['/*',...
'\begin{figure} ',...
'\begin{center} ',...
'\includegraphics[width=150mm]{images/seg1} ',...
'\end{center} ',...
'\caption{Segmentation results. ',...
'These are the results that can be obtained in the original space of the image ',...
'(i.e. the results that are not spatially normalised). ',...
'Top left: original image (X.img). ',...
'Top right: bias corrected image (mX.img). ',...
'Middle and bottom rows: segmented grey matter (c1X.img), ',...
'white matter (c2X.img) and CSF (c3X.img). \label{seg1}} ',...
'\end{figure} */'],...
['/*',...
'\begin{figure} ',...
'\begin{center} ',...
'\includegraphics[width=150mm]{images/seg2} ',...
'\end{center} ',...
'\caption{Segmentation results. ',...
'These are the spatially normalised results that can be obtained ',...
'(note that CSF data is not shown). ',...
'Top row: The tissue probability maps used to guide the segmentation. ',...
'Middle row: Spatially normalised tissue maps of grey and white matter ',...
'(wc1X.img and wc2X.img). ',...
'Bottom row: Modulated spatially normalised tissue maps of grey and ',...
'white matter (mwc1X.img and mwc2X.img). \label{seg2}} ',...
'\end{figure} */'],...
[...
'A deformation field is a vector field, where three values are associated with ',...
'each location in the field.  The field maps from co-ordinates in the ',...
'normalised image back to co-ordinates in the original image.  The value of ',...
'the field at co-ordinate [x y z] in the normalised space will be the ',...
'co-ordinate [x'' y'' z''] in the original volume. ',...
'The gradient of the deformation field at a co-ordinate is its Jacobian ',...
'matrix, and it consists of a 3x3 matrix:'],...
'',...
'%   /                      \',...
'%   | dx''/dx  dx''/dy dx''/dz |',...
'%   |                       |',...
'%   | dy''/dx  dy''/dy dy''/dz |',...
'%   |                       |',...
'%   | dz''/dx  dz''/dy dz''/dz |',...
'%   \                      /',...
['/* \begin{eqnarray*}',...
'\begin{pmatrix}',...
'\frac{dx''}{dx} & \frac{dx''}{dy} & \frac{dx''}{dz}\cr',...
'\frac{dy''}{dx} & \frac{dy''}{dy} & \frac{dy''}{dz}\cr',...
'\frac{dz''}{dx} & \frac{dz''}{dy} & \frac{dz''}{dz}\cr',...
'\end{pmatrix}\end{eqnarray*}*/'],...
[...
'The value of dx''/dy is a measure of how much x'' changes if y is changed by a ',...
'tiny amount. ',...
'The determinant of the Jacobian is the measure of relative volumes of warped ',...
'and unwarped structures.  The modulation step simply involves multiplying by ',...
'the relative volumes /*(see Figure \ref{seg2})*/.']};


%------------------------------------------------------------------------
% branches to select
%------------------------------------------------------------------------

estwrite      = branch('VBM: Estimate + Write','estwrite',{data,opts,output});
estwrite.prog   = @execute_estwrite;
estwrite.vfiles = @vfiles;
estwrite.help   = {[...
'Voxel based morphometry of cross-sectional data. ',...
'This function can be used for bias correcting, spatially normalising ',...
'or segmenting your data. In contrast to Johns original version this tool additionally ',...
'allows to apply a Hidden Markov Random Field approach to provide spatial constraints. ',...
'Furthermore, segmentations can be calculated without the use of priors. In this case ',...
'priors are only used for the normalisation step, but the bayesian rule to combine priors ',...
'with tissue type probabilities will be omitted. Thus, probabilities will not rely on priors, ',...
'which utilises segmentation of brains deviating from healthy, adult control population. ',...
'If no priors are used the segmented images are indicated by an "p" instead of "c". ',...
'Additionally, the voxel size of the segmentations can now be choosen manually. This allows ',...
'to write data with high spatial resolution which is useful to provide small voxel size in ',...
'comparison to size of gaussian smoothing filter making the data more independent from normal ',...
'distribution' ],...
'',...
[...
'Many investigators use tools within older versions of SPM for ',...
'a technique that has become known as "optimised" voxel-based ',...
'morphometry (VBM). ',...
'VBM performs region-wise volumetric comparisons among populations of subjects. ',...
'It requires the images to be spatially normalised, segmented into ',...
'different tissue classes, and smoothed, prior to performing ',...
'statistical tests/* \cite{wright_vbm,am_vbmreview,ashburner00b,john_should}*/. The "optimised" ',...
'pre-processing strategy involved spatially normalising subjects'' brain images to a ',...
'standard space, by matching grey matter in these images, to ',...
'a grey matter reference.  The historical motivation behind this ',...
'approach was to reduce the confounding effects of non-brain (e.g. scalp) ',...
'structural variability on the registration. ',...
'Tissue classification in older versions of SPM required the images to be registered ',...
'with tissue probability maps. After registration, these ',...
'maps represented the prior probability of different tissue classes ',...
'being found at each location in an image.  Bayes rule can ',...
'then be optionally used to combine these priors with tissue type probabilities ',...
'derived from voxel intensities, to provide the posterior probability.'],...
'',...
[...
'This procedure was inherently circular, because the ',...
'registration required an initial tissue classification, and the ',...
'tissue classification requires an initial registration.  This circularity ',...
'is resolved here by combining both components into a single ',...
'generative model. This model also includes parameters that account ',...
'for image intensity non-uniformity. ',...
'Estimating the model parameters (for a maximum a posteriori solution) ',...
'involves alternating among classification, bias correction and registration steps. ',...
'This approach provides better results than simple serial applications of each component.'],...
'',[...
'Note that multi-spectral segmentation (e.g. from a registered T1 and T2 image) is ',...
'not yet implemented, but is planned for a future SPM version.']};

%------------------------------------------------------------------------

write      = branch('VBM: Write already estimated segmentations','write',{data,output});
write.prog   = @execute_write;
write.vfiles = @vfiles;
write.help = {[...
'Allows previously estimated segmentations (stored in imagename''_sn.mat'' files) ',...
'to save the segmented images only without estimating the segmentation again. ',...
'This might be helpful if you have already estimated segmentations and you need ',...
'an additional tissue class, or you want to change voxel size of segmented images,'...
'or you want to apply HMRF or want to omit the use of priors.']};

%------------------------------------------------------------------------

subj      = branch('Subject','subj',{data});
subj.help   = {[...
'Select raw data (e.g. T1 images) for this subject. ',...
'The same warping parameters are used within subject.']};

%------------------------------------------------------------------------

datalong.type   = 'repeat';
datalong.name   = 'Data';
datalong.values = {subj};
datalong.num    = [1 Inf];
datalong.help   = {'List of subjects. Images of each subject are processed for longitudinal analysis.'};

%------------------------------------------------------------------------
% for longitudinal processing we only use unmodulated normalised
grey.labels = {...
    'None',...
    'Unmodulated Normalised'};
grey.values = {[0 0 0],[0 1 0]};

white.labels = grey.labels;
white.values = grey.values;
csf.labels = grey.labels;
csf.values = grey.values;

%------------------------------------------------------------------------

output     = branch('Writing options','output',{grey,white,csf,bias,extopts});

%------------------------------------------------------------------------

writelong      = branch('VBM: Write longitudinal data','writelong',{datalong,output});
writelong.prog   = @execute_writelong;
writelong.vfiles = @vfiles;
writelong.help = {[...
'This routine write segmented images for longitudinal analysis, where images ',...
'were acquired from the same subject to track changes over time. ',...
'Images of each subject are realigned to ',...
'each other, by aligning all scans from one subject to the ',...
'first scan.  Then the estimated warps of the first image ',...
'are applied to all images of one subject. The warping parameters should be ',...
'the same for all images, because we want to prevent, that ',...
'differences between the images are removed by intra-individual warps.']};

%------------------------------------------------------------------------
tools = cg_vbm_tools;
%------------------------------------------------------------------------

job.type = 'repeat';
job.name = 'VBM5.1';
job.tag  = 'vbm';
%job.values = {estwrite,write,writelong,tools};
job.values = {estwrite,write,tools};

return;

%------------------------------------------------------------------------

%------------------------------------------------------------------------
function print_opts(job,estopts)

if nargin == 1, estopts = 0; end

str = [];
if estopts
	str = [str struct('name', 'Gaussians:','value',sprintf('%d %d %d %d',job.opts.ngaus))];
	str = [str struct('name', 'Warp cutoff:','value',sprintf('%d',job.opts.warpco))];
	str = [str struct('name', 'Warp regularisation:','value',sprintf('%g',job.opts.warpreg))];
	str = [str struct('name', 'Bias regularisation:','value',sprintf('%g',job.opts.biasreg))];
	str = [str struct('name', 'Bias FWHM:','value',sprintf('%d',job.opts.biasfwhm))];
end
str = [str struct('name', 'Clean up:','value',sprintf('%d',job.output.extopts.cleanup))];
if (job.output.extopts.mrf==1)
	str = [str struct('name', 'Iterative HMRF:','value',sprintf('%g',0.1))];
else
	str = [str struct('name', 'HMRF:','value',sprintf('%g',job.output.extopts.mrf))];
end
str = [str struct('name', 'Write affine only:','value',sprintf('%d',job.output.extopts.writeaffine))];
str = [str struct('name', 'Use tissue priors:','value',sprintf('%d',job.output.extopts.usepriors))];
fprintf('Options:\n');
for i=1:size(str,2)
	fprintf('%30s\t%s\n', str(i).name ,str(i).value);
end
fprintf('\n');

return

%------------------------------------------------------------------------
function execute_estwrite(job)

print_opts(job,1)

job.opts.tpm = strvcat(job.opts.tpm{:});
if isfield(job.opts,'msk'),
    job.opts.msk = strvcat(job.opts.msk{:});
end;
for i=1:numel(job.data),
    res           = cg_vbm(job.data{i},job.opts);
    [sn,isn]   = spm_prep2sn(res);
    [pth,nam]     = spm_fileparts(job.data{i});
    savefields(fullfile(pth,[nam '_seg_sn.mat']),sn);
    savefields(fullfile(pth,[nam '_seg_inv_sn.mat']),isn);
	cg_vbm_write(sn,job,isn);
end;
return;
%------------------------------------------------------------------------

%------------------------------------------------------------------------
function execute_write(job)

print_opts(job)

for i=1:numel(job.data),
    [pth,nam]     = spm_fileparts(job.data{i});
    sn_name = fullfile(pth,[nam '_seg_sn.mat']);
    if exist(sn_name), sn = load(sn_name);
    else
       	error(['Can''t load file ' sn_name]);    
       	return    
    end
    % use selected data rather than saved data in sn
    sn.VF = spm_vol(char(job.data{i}));
	% use default TPM's if file does not exist
    if ~exist(sn.VG(1).fname)
		global defaults
		sn.VG = spm_vol(defaults.preproc.tpm);
	end
    isn_name = fullfile(pth,[nam '_seg_inv_sn.mat']);
   	if exist(isn_name), isn = load(isn_name);
   	else
   		error(['Can''t load file ' isn_name]);    
   		return    
   	end
    cg_vbm_write(sn,job,isn);
end;
return;
%------------------------------------------------------------------------

%------------------------------------------------------------------------
function execute_writelong(job)

for i=1:length(job.subj),

	% realign to first scan
	P             = {};
	for j=1:length(job.subj(i).data),
		P{j}  = strvcat(job.subj(i).data{j});
	end;
	flags.quality = 1;
	flags.fwhm    = 5;
	flags.sep     = 4;
	flags.rtm     = 0;
	flags.PW      = '';
	flags.interp  = 2;
	flags.wrap    = [0 0 0];
	spm_realign(P,flags);

	% write segmentations for baseline scan
    [pth,nam]     = spm_fileparts(job.subj(i).data{1});
    sn_name = fullfile(pth,[nam '_seg_sn.mat']);
    if exist(sn_name), sn1 = load(sn_name);
    else
       	error(['Can''t load file ' sn_name]);    
       	return    
    end
	% use default TPM's if file does not exist
    if ~exist(sn1.VG(1).fname)
		global defaults
		sn1.VG = spm_vol(defaults.preproc.tpm);
	end
	if job.output.extopts.writeaffine
	    isn_name = fullfile(pth,[nam '_seg_inv_sn.mat']);
    	if exist(isn_name), isn1 = load(isn_name);
    	else
       		error(['Can''t load file ' isn_name]);    
       		return    
    	end
	    cg_vbm_write(sn1,job,isn1);
    else
	    cg_vbm_write(sn1,job1);
    end

	% write segmentations for subsequent scans
	for j=2:length(job.subj(i).data),
    	[pth,nam]     = spm_fileparts(job.subj(i).data{j})
	    sn_name = fullfile(pth,[nam '_seg_sn.mat']);
    	if exist(sn_name), sn = load(sn_name);
	    else
    	   	error(['Can''t load file ' sn_name]);    
       		return    
	    end
	    % use priors and warps of baseline scan
	    sn.VG = sn1.VG;
	    sn.Affine = sn1.Affine;
	    sn.Tr = sn1.Tr;
	    sn.flags.Affine = sn1.flags.Affine;
	    sn.flags.Twarp = sn1.flags.Twarp;
	    sn.flags.Tbias = sn1.flags.Tbias;
		if job.output.extopts.writeaffine
		    isn_name = fullfile(pth,[nam '_seg_inv_sn.mat']);
    		if exist(isn_name), isn = load(isn_name);
	    	else
    	   		error(['Can''t load file ' isn_name]);    
       			return    
	    	end
	    	isn.Affine = isn1.Affine;
		    cg_vbm_write(sn,job,isn);
	    else
		    cg_vbm_write(sn,job);
    	end
	end
end;
return;
%------------------------------------------------------------------------

%------------------------------------------------------------------------
function savefields(fnam,p)

if length(p)>1, error('Can''t save fields.'); end;
fn = fieldnames(p);
if numel(fn)==0, return; end;
for i=1:length(fn),
    eval([fn{i} '= p.' fn{i} ';']);
end;
if spm_matlab_version_chk('7') >= 0
    save(fnam,'-V6',fn{:});
else
    save(fnam,fn{:});
end;

return;
%------------------------------------------------------------------------

%------------------------------------------------------------------------
function vf = vfiles(job)

opts  = job.output;

% use old syntax for compatibility purposes
if isfield(opts.BIAS,'descalp')
	opts.BIAS = [opts.BIAS.native opts.BIAS.warped opts.BIAS.descalp];
end

if isfield(job,'subj')
	longstr = '_long'
else
	longstr = '';
end
sopts = [[opts.GM.modulated opts.GM.warped opts.GM.native] ;...
		[opts.WM.modulated opts.WM.warped opts.WM.native] ;...
		[opts.CSF.modulated opts.CSF.warped opts.CSF.native]];
vf    = cell(numel(job.data),2);
for i=1:numel(job.data),
    [pth,nam,ext,num] = spm_fileparts(job.data{i});
    vf{i,1} = fullfile(pth,[nam '_seg_sn.mat']);
    vf{i,2} = fullfile(pth,[nam '_seg_inv_sn.mat']);

	if opts.extopts.writeaffine
	   if isempty(strfind(nam,'_affine'))
       		nam = [nam '_affine'];
       end
	end
    j       = 3;
    if opts.BIAS(3)
    	descalp = '_descalped';
    else descalp = '';end
    if opts.BIAS(1),
        vf{i,j} = fullfile(pth,['m' nam descalp ext ',1']);
        j       = j + 1;
    end;
    if opts.BIAS(2),
        vf{i,j} = fullfile(pth,['wm' nam descalp ext ',1']);
        j       = j + 1;
    end;
    if opts.extopts.mrf
    	if opts.extopts.mrf == 1
	        cnam = [nam longstr '_aMRF'];
	    else
	        cnam = [nam longstr '_HMRF'];
	    end
    else
        cnam = [nam longstr];
    end;
    for k1=1:3,
        if sopts(k1,3),
		    if opts.extopts.usepriors
	            vf{i,j} = fullfile(pth,[  'c', num2str(k1), cnam, ext, ',1']);
	        else
	            vf{i,j} = fullfile(pth,[  'p', num2str(k1), cnam, ext, ',1']);
	        end
            j       = j + 1;
        end;
        if sopts(k1,2),
		    if opts.extopts.usepriors
	            vf{i,j} = fullfile(pth,[  'wc', num2str(k1), cnam, ext, ',1']);
	        else
	            vf{i,j} = fullfile(pth,[  'wp', num2str(k1), cnam, ext, ',1']);
	        end
            j       = j + 1;
        end;
        if sopts(k1,1)==1,
		    if opts.extopts.usepriors
	            vf{i,j} = fullfile(pth,[  'mwc', num2str(k1), cnam, ext, ',1']);
	        else
	            vf{i,j} = fullfile(pth,[  'mwp', num2str(k1), cnam, ext, ',1']);
	        end
            j       = j + 1;
        end;
        if sopts(k1,1)==2,
		    if opts.extopts.usepriors
	            vf{i,j} = fullfile(pth,[  'm0wc', num2str(k1), cnam, ext, ',1']);
	        else
	            vf{i,j} = fullfile(pth,[  'm0wp', num2str(k1), cnam, ext, ',1']);
	        end
            j       = j + 1;
        end;
    end;
end;
vf = vf(:);

return;
